
package monitor;

/**
 *  Classe Monitor.
 *  @author Cesare Stefanelli e Antonio Corradi.
 *  @version Monitor 1.0 
 */
public class Monitor {

    	private  int inTransito [];
    	private  int inAttesa [];
	Mutex lock = new Mutex();
	Urgent urgentQueue = new Urgent();
    	private  VarCond VC[] = new VarCond [3];

	public Monitor() {
		int i;
		inTransito = new int [3];
		inAttesa = new int [3];

		for (i=0; i<3; i++) inTransito[i] = 0;
		for (i=0; i<3; i++) inAttesa[i] = 0;
		for (i=0; i<3; i++) VC[i] = new VarCond(lock, urgentQueue);
	}
	
	private void entraMonitor() {
		lock.MutexWait();
	}
	
	private void esciMonitor() {
		if (urgentQueue.queue()) 
			urgentQueue.urgentSignal();
		else 
			lock.MutexSignal();
	}


	/**
 	 *  Accesso al Monitor. IMPORTANTE: chiamare SEMPRE entraMonitor (metodo private) come prima istruzione
	 *  ed esciMonitor (metodo private) come ultima.
       *
	 *  @param tipo: il tipo del processo in ingresso.
	 */
   	public void richiestaAccesso(int tipo) {
	  	entraMonitor();
        	while (Condizione(tipo)) {
			inAttesa[tipo]++;
			VC[tipo].varCondWait(); 
			inAttesa[tipo]--;
        	}
        	inTransito[tipo]++;
	  	show();
	  	VC[tipo].varCondSignal();
		esciMonitor();
    	}

	/**
 	 *  Uscita dal Monitor. IMPORTANTE: chiamare SEMPRE entraMonitor come prima istruzione
	 *  ed esciMonitor come ultima.
	 *  @parameter tipo e' il tipo del processo in ingresso
	 */
    	public void uscita(int tipo) {
	  	entraMonitor();
        	inTransito[tipo]--;
	  	show();
     	  	if(inTransito[tipo]==0) {
		  	VC[0].varCondSignal();
		  	VC[1].varCondSignal();
		  	VC[2].varCondSignal();
	  	} 
		 esciMonitor();
	}

    	private void show() {
	  	int i;
  	  	System.out.print("Navi nel canale: ");

        	for(i=0;i<3;i++) {
	   		if (i==0) System.out.print(inTransito[i] + " Passeggeri, ");
	   		if (i==1) System.out.print(inTransito[i] + " Petroliere e ");
	   		if (i==2) System.out.print(inTransito[i] + " Merci");
	 	}
	  	System.out.println( " ("+inAttesa[0]+" "+inAttesa[1]+" "+inAttesa[2] + ")");
    	}

    	private boolean Condizione (int tipo) {
        	if ( (tipo == 0 && inTransito[1]!= 0) || 
		 	(tipo == 1 && inTransito[0]!= 0) || 
		 	(tipo == 1 && inTransito[2]!= 0) || 
		 	(tipo == 1 && inAttesa[0]!=0) ||
		 	(tipo == 2 && inTransito[1]!= 0) || 
		 	(tipo == 2 && inAttesa[0]!=0) ||
		 	(tipo == 2 && inAttesa[1]!=0)
		 
	     	) {
			return true;
	  	} else {
			return false;
	  	}
    	}
}

