package monitor;

/**
 *  Classe Variabili Condizione.
 *  @author Cesare Stefanelli e Antonio Corradi.
 *  @version Monitor 1.0 
 */
public class VarCond {
	Mutex lock;
	Urgent urgentQueue;
	int attesa;

	VarCond(Mutex lockMonitor, Urgent urgentMonitor){
		this.lock = lockMonitor;
		this.urgentQueue = urgentMonitor;
	}

	/** 
	 * Wait sulla Var Condizione, prima della sospensione rilascia il Mutex sul Monitor. 
	 */
	synchronized public void varCondWait() {
		attesa ++;
		lock.MutexSignal();
		try{ 
			wait(); 
		} catch(InterruptedException e){}
		attesa --;
	}

	synchronized private boolean varCondSignalNascosta() {
		if (attesa > 0) {
			notify(); 
			return true;
		} else 
			return false;

	}

	/** 
	 * Signal sulla Var Condizione: Signal sospensiva, sospensione processo su Coda Urgent.
	 */
	public void varCondSignal() {
		if (varCondSignalNascosta()) urgentQueue.urgentWait();
	}

	/** 
	 * Verifica la presenza di processi sospesi sulla Variabile Condizione.
	 * @return ritorna true se ci sono processi sospesi, false altrimenti
	 */
	synchronized public boolean queue() {
		if (attesa > 0) return true;
		else return false;
	}


}


